import java.lang.Math;

/* Class: MinHeap 
 * Coda di priorità minima
 * 
 */

public class MinHeap {
	private Nodo [] array;
	private int heap_size,n;
	
	private int right(int i) {return i*2+2;}
	private int left(int i) {return i*2+1;}
	private int padre(int i) {return (int)Math.floor((i-1)/2);}
	
	public MinHeap(int l) {
		n=0;
		array=new Nodo[l];
	}
	
/* Method: MinHeap 
 * Costruisce la coda di priorità partendo da un array di Nodi
 * 
 */


	public MinHeap(Nodo[] x,int n) {
		this.n=n;
		array=x;
		
		BuildMinHeap();	
	}

/* Method: Inserisci
 * Inserisce un nuovo Nodo nel MinHeap, passando come parametri "Carattere" e "Frequenza"
 * 
 */

	
	public void Inserisci(char c,int f) {
		array[n]=new Nodo(c,f);
		n++;
	}

/* Method: Inserisci 
 * Inserisce un nuovo Nodo nel MinHeap, passando come parametro il Nodo
 * 
 */

	public void Inserisci(Nodo x) {
		array[n]=x;
		n++;
	}
	
/*Method: heapify 
 * Metodo che riordina la coda affinchè siano rispettate le condizione di ordinamento per priorità
 * 
 */

	public void heapify(int x) {
        
        int l=left(x);
        int r=right(x);
        int min=x;
        if(l<heap_size && array[l].getFreq()<array[min].getFreq()) min=l;
        if(r<heap_size && array[r].getFreq()<array[min].getFreq()) min=r;
        
        if(min!=x) {
            scambia(x,min);
            heapify(min);
        }
    }
	
	public void scambia(int i,int j) {
		Nodo aux=array[i];
		array[i]=array[j];
		array[j]=aux;
	}
	
	
	public void BuildMinHeap() {
		heap_size=n;
		for(int i=padre(n);i>=0;i--) heapify(i);
	}
	
	public void stampa() {
		for(int i=0;i<n;i++) {
			array[i].Stampa_Nodo();
		}
	}
	
/*Method: Extract 
 * Estrae il Nodo in cima alla coda di priorità
 * 
 */

	public Nodo Extract() {
		Nodo ret=null;
		if(heap_size>0) {
			ret=array[0];
			
			array[0]=array[heap_size-1];
			heap_size-=1;
			if(heap_size>0)heapify(0);
		}
		return ret;
	}

/* Method: Enqueue 
 * Accoda un nuovo Nodo nel MinHeap
 * 
 */
	

	public void Enqueue(Nodo x){

		array[heap_size]=x;
        int i=heap_size;
		while(i>=0 && array[padre(i)].getFreq()>array[i].getFreq()){
            scambia(i,padre(i));
            i=padre(i);
        }
        heap_size+=1;
 
    }
/*Method: Increase_key 
 * Setta il valore di un Nodo e lo ricolloca nella coda mantenendone le proprietà di ordinamento per priorità
 * 
 */

    public void Increase_key(int i,Nodo key){
        array[i]=key;
        while(i>=0 && array[padre(i)].getFreq()>array[i].getFreq()){
            scambia(i,padre(i));
            i=padre(i);
        }
    }
/* Method: getSize 
 * Restituisce la dimensione del MinHeap
 * 
 */

	public int getSize() {return heap_size;}
}
